@@ -389,7 +389,9 @@ eng_cat = function(options) {
 
   if (is.null(lang <- options$engine.opts$lang) && is.null(lang <- options$class.source))
     return('')
-  options$engine = lang
+  # Use engine to set the attribute
+  options$engine = lang[1]
+  options$class.source = setdiff(options$class.source, lang[1])
   engine_output(options, options$code, NULL)
 }
 
@@ -774,6 +776,43 @@ eng_targets = function(options) {
   targets::tar_engine_knitr(options = options)
 }
 
+# a comment engine to return nothing
+eng_comment = function(options) {}
+
+## a verbatim engine that returns its chunk content verbatim
+eng_verbatim = function(options) {
+  # change default for the cat engine
+  options$eval = FALSE
+  options = set_lang(options)
+  eng_cat(options)
+}
+
+set_lang = function(options, default = 'default') {
+  # specify the lang name in engine.opts = list(lang = ), or lang/language,
+  # or class.source; if all are empty, use 'default'
+  options$engine.opts$lang = options$engine.opts$lang %n%
+    unlist(options[c('lang', 'language')])[1] %n%
+    options$class.source %n% default
+  options
+}
+
+# embed a file verbatim
+eng_embed = function(options) {
+  # if `file` is empty, use `code` as the list of files
+  if (is.null(f <- options$file)) {
+    f = gsub('^["\']|["\']$', '', options$code)  # in case paths are quoted
+    if (length(f) == 0) return()
+    options$code = xfun::read_utf8(f)  # TODO: use xfun::read_all()
+  }
+  # use the filename extension as the default language name
+  if (nchar(lang <- file_ext(f[1])) > 1) {
+    lang = sub('^R', '', lang)  # Rmd -> md, Rhtml -> html, etc.
+    if (lang == 'nw') lang = 'tex'
+  }
+  options = set_lang(options, tolower(lang))
+  eng_verbatim(options)
+}
+
 # set engines for interpreted languages
 local({
   for (i in c(
@@ -785,12 +824,34 @@ local({
 
 # additional engines
 knit_engines$set(
-  highlight = eng_highlight, Rcpp = eng_Rcpp, tikz = eng_tikz, dot = eng_dot,
-  c = eng_shlib, cc = eng_shlib, fortran = eng_shlib, fortran95 = eng_shlib, asy = eng_dot,
-  cat = eng_cat, asis = eng_asis, stan = eng_stan, block = eng_block,
-  block2 = eng_block2, js = eng_js, css = eng_css, sql = eng_sql, go = eng_go,
-  python = eng_python, julia = eng_julia, sass = eng_sxss, scss = eng_sxss, R = eng_r,
-  bslib = eng_bslib, targets = eng_targets
+  asis = eng_asis,
+  asy = eng_dot,
+  block = eng_block,
+  block2 = eng_block2,
+  bslib = eng_bslib,
+  c = eng_shlib,
+  cat = eng_cat,
+  cc = eng_shlib,
+  comment = eng_comment,
+  css = eng_css,
+  dot = eng_dot,
+  embed = eng_embed,
+  fortran = eng_shlib,
+  fortran95 = eng_shlib,
+  go = eng_go,
+  highlight = eng_highlight,
+  js = eng_js,
+  julia = eng_julia,
+  python = eng_python,
+  R = eng_r,
+  Rcpp = eng_Rcpp,
+  sass = eng_sxss,
+  scss = eng_sxss,
+  sql = eng_sql,
+  stan = eng_stan,
+  targets = eng_targets,
+  tikz = eng_tikz,
+  verbatim = eng_verbatim
 )
 
 cache_engines$set(python = cache_eng_python)
